@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2015 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Device for first/last page handling device */
@@ -46,6 +46,7 @@
 #include "gsstype.h"
 #include "gdevprn.h"
 #include "gdevp14.h"        /* Needed to patch up the procs after compositor creation */
+#include "gximage.h"        /* For gx_image_enum */
 #include "gdevsclass.h"
 #include "gdevflp.h"
 #include <stdlib.h>
@@ -111,7 +112,7 @@ static dev_proc_process_page(flp_process_page);
 
 #define public_st_flp_device()	/* in gsdevice.c */\
   gs_public_st_complex_only(st_flp_device, gx_device, "first_lastpage",\
-    0, flp_enum_ptrs, flp_reloc_ptrs, gx_device_finalize)
+    0, flp_enum_ptrs, flp_reloc_ptrs, default_subclass_finalize)
 
 static
 ENUM_PTRS_WITH(flp_enum_ptrs, gx_device *dev);
@@ -269,7 +270,7 @@ static int ParsePageList(gx_device *dev, first_last_subclass_data *psubclass_dat
                 }
             }
             /* str should now point to the last page number (we hope!) */
-            LastPage = atoi(str);
+            psubclass_data->LastListPage = LastPage = atoi(str);
 
             psubclass_data->PageArraySize = (LastPage + 7) / 8;
             psubclass_data->PageArray = gs_alloc_bytes(dev->memory->non_gc_memory, psubclass_data->PageArraySize, "array of pages selected");
@@ -305,6 +306,10 @@ static int ParsePageList(gx_device *dev, first_last_subclass_data *psubclass_dat
                         LastPage = 0;
 
                     for (i=Page; i<= LastPage;i++) {
+                        if (i > psubclass_data->LastListPage - 1) {
+                            emprintf(dev->memory, "\n**** Error : rangecheck processing PageList\n");
+                            return_error(gs_error_rangecheck);
+                        }
                         byte = (int)(i / 8);
                         bit = i % 8;
                         c = 0x01 << bit;
@@ -314,6 +319,10 @@ static int ParsePageList(gx_device *dev, first_last_subclass_data *psubclass_dat
                     Page = atoi(oldstr) - 1;
                     if (Page < 0)
                         Page = 0;
+                    if (Page > psubclass_data->LastListPage - 1) {
+                        emprintf(dev->memory, "\n**** Error : rangecheck processing PageList\n");
+                        return_error(gs_error_rangecheck);
+                    }
                     byte = (int)(Page / 8);
                     bit = Page % 8;
                     c = 0x01 << bit;
@@ -345,19 +354,23 @@ static int SkipPage(gx_device *dev)
     }
 
     if (psubclass_data->PageArray) {
-        if (psubclass_data->FromToEnd != 0 && psubclass_data->PageCount >= psubclass_data->FromToEnd)
+        if (psubclass_data->FromToEnd != 0 && psubclass_data->PageCount >= psubclass_data->FromToEnd - 1)
             return 0;
         else {
-            int byte, bit;
-            char c;
-
-            byte = (int)((psubclass_data->PageCount) / 8);
-            bit = (psubclass_data->PageCount) % 8;
-            c = 0x01 << bit;
-            if (((char *)psubclass_data->PageArray)[byte] & c)
-                return 0;
-            else
+            if (psubclass_data->PageCount > psubclass_data->LastListPage - 1)
                 return 1;
+            else {
+                int byte, bit;
+                char c;
+
+                byte = (int)((psubclass_data->PageCount) / 8);
+                bit = (psubclass_data->PageCount) % 8;
+                c = 0x01 << bit;
+                if (((char *)psubclass_data->PageArray)[byte] & c)
+                    return 0;
+                else
+                    return 1;
+            }
         }
     } else {
         if (psubclass_data->EvenOdd != none) {
@@ -730,6 +743,8 @@ int flp_strip_copy_rop(gx_device *dev, const byte *sdata, int sourcex, uint sras
 
 typedef struct flp_image_enum_s {
     gx_image_enum_common;
+    int y;
+    int height;
 } flp_image_enum;
 gs_private_st_composite(st_flp_image_enum, flp_image_enum, "flp_image_enum",
 flp_image_enum_enum_ptrs, flp_image_enum_reloc_ptrs);
@@ -750,7 +765,14 @@ flp_image_plane_data(gx_image_enum_common_t * info,
                      const gx_image_plane_t * planes, int height,
                      int *rows_used)
 {
-    return 0;
+    flp_image_enum *pie = (flp_image_enum *)info;
+
+    pie->y += height;
+    *rows_used = height;
+
+    if (pie->y < pie->height)
+        return 0;
+    return 1;
 }
 
 static int
@@ -800,6 +822,8 @@ int flp_begin_typed_image(gx_device *dev, const gs_gstate *pgs, const gs_matrix
                         (gx_device *)dev, num_components, pim->format);
     pie->memory = memory;
     pie->skipping = true;
+    pie->height = pim->Height;
+    pie->y = 0;
 
     return 0;
 }
@@ -822,6 +846,14 @@ int flp_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_
 {
     int code = SkipPage(dev);
 
+    /* The returned pointer 'pcdev' is a signal to the caller, if its not the same as the current
+     * device in the graphics state, then it gets set as the current device (basically if its not the
+     * same its a signal that we want to push a new compositor device to the head). So here we start
+     * by making sure that its set to the one in the graphics state. If the child device wants a
+     * compositor it will overwrite this value.
+     */
+    *pcdev = pgs->device;
+
     if (code < 0)
         return code;
     if (!code)
