@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2015 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Derived from gdevflp.c */
@@ -31,6 +31,7 @@
 #include "gsstype.h"
 #include "gdevprn.h"
 #include "gdevp14.h"        /* Needed to patch up the procs after compositor creation */
+#include "gximage.h"        /* For gx_image_enum */
 #include "gdevsclass.h"
 #include "gdevoflt.h"
 
@@ -73,7 +74,7 @@ static dev_proc_strip_tile_rect_devn(obj_filter_strip_tile_rect_devn);
 
 #define public_st_obj_filter_device()	/* in gsdevice.c */\
   gs_public_st_complex_only(st_obj_filter_device, gx_device, "object filter",\
-    0, obj_filter_enum_ptrs, obj_filter_reloc_ptrs, gx_device_finalize)
+    0, obj_filter_enum_ptrs, obj_filter_reloc_ptrs, default_subclass_finalize)
 
 static
 ENUM_PTRS_WITH(obj_filter_enum_ptrs, gx_device *dev);
@@ -311,6 +312,8 @@ int obj_filter_strip_copy_rop(gx_device *dev, const byte *sdata, int sourcex, ui
 
 typedef struct obj_filter_image_enum_s {
     gx_image_enum_common;
+    int y;
+    int height;
 } obj_filter_image_enum;
 gs_private_st_composite(st_obj_filter_image_enum, obj_filter_image_enum, "obj_filter_image_enum",
   obj_filter_image_enum_enum_ptrs, obj_filter_image_enum_reloc_ptrs);
@@ -331,7 +334,14 @@ obj_filter_image_plane_data(gx_image_enum_common_t * info,
                      const gx_image_plane_t * planes, int height,
                      int *rows_used)
 {
-    return 0;
+    obj_filter_image_enum *pie = (obj_filter_image_enum *)info;
+
+    pie->y += height;
+    *rows_used = height;
+
+    if (pie->y < pie->height)
+        return 0;
+    return 1;
 }
 
 static int
@@ -378,6 +388,8 @@ int obj_filter_begin_typed_image(gx_device *dev, const gs_gstate *pgs, const gs_
                         (gx_device *)dev, num_components, pim->format);
     pie->memory = memory;
     pie->skipping = true;
+    pie->height = pim->Height;
+    pie->y = 0;
 
     return 0;
 }
