@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -23,9 +23,9 @@
 #include "gsiparm2.h"
 
 /* View clipping */
-int gs_initviewclip(gs_state *);
-int gs_eoviewclip(gs_state *);
-int gs_viewclip(gs_state *);
-int gs_viewclippath(gs_state *);
+int gs_initviewclip(gs_gstate *);
+int gs_eoviewclip(gs_gstate *);
+int gs_viewclip(gs_gstate *);
+int gs_viewclippath(gs_gstate *);
 
 #endif /* gsdps_INCLUDED */
