@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -23,6 +23,7 @@
 #include "gscsel.h"
 #include "gxfmap.h"
 #include "gscspace.h"
+#include "gsdcolor.h"
 
 #ifndef gx_device_DEFINED
 #  define gx_device_DEFINED
@@ -279,4 +280,22 @@ void cmap_transfer(gx_color_value *pconc, const gs_gstate * pgs,
                    gx_device * dev);
 void cmap_transfer_plane(gx_color_value *pconc, const gs_gstate *pgs,
                     gx_device *dev, int plane);
+
+typedef struct gx_cmapper_data_s gx_cmapper_data;
+
+typedef void (gx_cmapper_fn)(gx_cmapper_data *data);
+
+struct gx_cmapper_data_s {
+    gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
+    const gs_gstate *pgs;
+    gx_device *dev;
+    gs_color_select_t select;
+    gx_device_color devc;
+    gx_cmapper_fn *set_color;
+};
+
+gx_cmapper_fn *gx_get_cmapper(gx_cmapper_data *data, const gs_gstate *pgs,
+                              gx_device *dev, bool has_transfer, bool has_halftone,
+                              gs_color_select_t select);
+
 #endif /* gxcmap_INCLUDED */
