@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -313,6 +313,18 @@ dev_proc_strip_copy_rop2(gx_default_strip_copy_rop2);
 dev_proc_strip_tile_rect_devn(gx_default_strip_tile_rect_devn);
 dev_proc_copy_alpha_hl_color(gx_default_copy_alpha_hl_color);
 dev_proc_process_page(gx_default_process_page);
+dev_proc_begin_transparency_group(gx_default_begin_transparency_group);
+dev_proc_end_transparency_group(gx_default_end_transparency_group);
+dev_proc_begin_transparency_mask(gx_default_begin_transparency_mask);
+dev_proc_end_transparency_mask(gx_default_end_transparency_mask);
+dev_proc_discard_transparency_layer(gx_default_discard_transparency_layer);
+dev_proc_pattern_manage(gx_default_pattern_manage);
+dev_proc_push_transparency_state(gx_default_push_transparency_state);
+dev_proc_pop_transparency_state(gx_default_pop_transparency_state);
+dev_proc_put_image(gx_default_put_image);
+dev_proc_copy_alpha_hl_color(gx_default_no_copy_alpha_hl_color);
+dev_proc_copy_planes(gx_default_copy_planes);
+
 /* BACKWARD COMPATIBILITY */
 #define gx_non_imaging_create_compositor gx_null_create_compositor
 
@@ -650,16 +662,18 @@ void gx_device_request_leadingedge(gx_device *dev, int le_req);
 int gs_is_pdf14trans_compositor(const gs_composite_t * pct);
 
 #define subclass_common\
-    t_dev_proc_create_compositor *saved_compositor_method
+    t_dev_proc_create_compositor *saved_compositor_method;\
+    gx_device_forward *forwarding_dev
 
 typedef int (t_dev_proc_create_compositor) (gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte, gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev);
 
 typedef struct {
     t_dev_proc_create_compositor *saved_compositor_method;
+    gx_device_forward *forwarding_dev;
 } generic_subclass_data;
 
 
-int gx_copy_device_procs(gx_device_procs *dest_procs, gx_device_procs *src_procs, gx_device_procs *prototype_procs);
+int gx_copy_device_procs(gx_device *dest, gx_device *src, gx_device *prototype);
 int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, unsigned int private_data_size);
 int gx_device_unsubclass(gx_device *dev);
 int gx_update_from_subclass(gx_device *dev);
