@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -107,6 +107,7 @@ typedef struct gx_transfer_s {
                 /* dictionaries.  (In Level 1 systems, set_transfer and */\
                 /* effective_transfer are always the same.) */\
         gx_transfer set_transfer;		/* members are (RC) */\
+        int effective_transfer_non_identity_count;\
         gx_transfer_map *effective_transfer[GX_DEVICE_COLOR_MAX_COMPONENTS]; /* see below */\
 \
                 /* Color caches: */\
@@ -256,25 +257,6 @@ typedef struct gs_font_s gs_font;
 typedef struct gs_device_filter_stack_s gs_device_filter_stack_t;
 #endif
 
-/* Device filter stack structure is defined here so that gstate
-   lifecycle operations can access reference count; implementation is
-   in gsdfilt.c.
- */
-
-#ifndef gs_device_filter_DEFINED
-#  define gs_device_filter_DEFINED
-typedef struct gs_device_filter_s gs_device_filter_t;
-#endif
-
-/* This is the base structure from which device filters are derived. */
-struct gs_device_filter_stack_s {
-    gs_device_filter_stack_t *next;
-    gs_device_filter_t *df;
-    gx_device *next_device;
-    rc_header rc;
-};
-
-
 /* Define the graphics state structure itself. */
 /*
  * Note that the ctm member is a gs_matrix_fixed.  As such, it cannot be
@@ -371,6 +353,7 @@ struct gs_gstate_s {
     float textrise;
     float wordspacing;
     float texthscaling;
+    float PDFfontsize;
     gs_matrix textlinematrix;
     gs_matrix textmatrix;
     /* Current colors (non-stroking, and stroking) */ 
@@ -388,7 +371,6 @@ struct gs_gstate_s {
     /* Other stuff: */ 
     int level;			/* incremented by 1 per gsave */ 
     gx_device *device; 
-    gs_device_filter_stack_t *dfilter_stack; 
     gs_gstate_client_procs client_procs;
 };
 
@@ -454,7 +436,7 @@ struct_proc_finalize(gs_gstate_finalize);
 
 /*
  * Enumerate the pointers in a graphics state
- * except device and dfilter_stack which must
+ * except device which must
  * be handled specially.
  */
 #define gs_gstate_do_ptrs(m)\
@@ -481,11 +463,11 @@ struct_proc_finalize(gs_gstate_finalize);
 
 #define gs_gstate_num_ptrs 20
 
-/* The '+2' in the following is because gs_gstate.device
- * and gs_gstate.dfilter_stack are handled specially
+/* The '+1' in the following is because gs_gstate.device
+ * is handled specially
  */
 #define st_gs_gstate_num_ptrs\
-  (st_line_params_num_ptrs + st_cr_state_num_ptrs + gs_gstate_num_ptrs + 2)
+  (st_line_params_num_ptrs + st_cr_state_num_ptrs + gs_gstate_num_ptrs + 1)
 
 /* Initialize an graphics state, other than the parts covered by */
 /* gs_gstate_initial. */
