@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -82,9 +82,6 @@ gs_image_class_1_simple(gx_image_enum * penum)
                     penum->line = gs_alloc_bytes(penum->memory,
                                             penum->line_size, "image line");
                     if (penum->line == 0) {
-                        gx_default_end_image(penum->dev,
-                                             (gx_image_enum_common_t *)penum,
-                                             false);
                         return 0;
                     }
                 }
@@ -322,7 +319,7 @@ image_simple_expand(byte * line, int line_x, uint raster,
     /* Extreme negative values of x_extent cause the xl0 calculation
      * to explode. Workaround this here. */
     if (x_extent < min_int + 0x100)
-      x_extent = min_int + 0x100;
+      x_extent += 0x100;
 
     /* Set up the DDAs. */
     xl0 =
@@ -330,6 +327,9 @@ image_simple_expand(byte * line, int line_x, uint raster,
          fixed_fraction(fixed_pre_pixround(xcur)) :
          fixed_fraction(fixed_pre_pixround(xcur + x_extent)) - x_extent);
     xl0 += int2fixed(line_x);
+    /* We should never get a negative x10 here. If we do, all bets are off. */
+    if (xl0 < 0)
+        xl0 = 0, x_extent = 0;
     dda_init(xl, xl0, x_extent, w);
     dxx4 = xl.step;
     dda_step_add(dxx4, xl.step);
