@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2014 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -984,6 +984,15 @@ static int pscompact_isint(pscompstate *psc, int *i)
     }
     psc->bufferin[psc->inpos] = 0;
     *i = atoi(psc->bufferin);
+
+    /* Check for 32bit overflow */
+    if (psc->inpos > 9) {
+        char *end;
+        double d = strtod(psc->bufferin, &end);
+        if (d != (double)(int)*i)
+            return 0;
+    }
+
     return 1;
 }
 
@@ -1256,7 +1265,7 @@ static unsigned long pscompact_getcompactedblock(pscompstate *psc, unsigned char
                         psc->state = PSC_BufferOut;
                         break;
                     }
-                    if (psc->binary && psc->names && pscompact_isname(psc, &i)) {
+                    if (psc->names && pscompact_isname(psc, &i)) {
                         /* Encode as a name lookup */
                         if (i >= 0) {
                             /* Executable */
@@ -1407,8 +1416,9 @@ static unsigned long pscompact_getcompactedblock(pscompstate *psc, unsigned char
                             pscompact_copyinout_bin(psc);
                             break;
                         } else if (psc->inpos < 65536) {
-                            pscompact_bufferatstart(psc, psc->inpos>>8);
-                            pscompact_bufferatstart(psc, psc->inpos & 255);
+                            int count = psc->inpos;
+                            pscompact_bufferatstart(psc, count>>8);
+                            pscompact_bufferatstart(psc, count & 255);
                             pscompact_bufferatstart(psc, 144);
                             pscompact_copyinout_bin(psc);
                             break;
@@ -1510,8 +1520,9 @@ static unsigned long pscompact_getcompactedblock(pscompstate *psc, unsigned char
                             pscompact_bufferatstart(psc, 142);
                             pscompact_copyinout_bin(psc);
                         } else if (psc->inpos < 65536) {
-                            pscompact_bufferatstart(psc, psc->inpos>>8);
-                            pscompact_bufferatstart(psc, psc->inpos & 255);
+                            int count = psc->inpos;
+                            pscompact_bufferatstart(psc, count>>8);
+                            pscompact_bufferatstart(psc, count & 255);
                             pscompact_bufferatstart(psc, 144);
                             pscompact_copyinout_bin(psc);
                         } else {
@@ -1572,7 +1583,7 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
     unsigned long psc_len;
     pscompstate psc = { 0 };
     unsigned long numfiles = 0;
-    char **foundfiles = NULL;
+    char **foundfiles = NULL, *temp;
 
     prefixed_path = malloc(PATH_STR_LEN);
     found_path = malloc(PATH_STR_LEN);
@@ -1611,7 +1622,18 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
             continue;
 
         numfiles++;
-        foundfiles = realloc(foundfiles, sizeof(char *) * numfiles);
+        temp = realloc(foundfiles, sizeof(char *) * numfiles);
+        if (temp == NULL) {
+            free(cbuf);
+            free(ubuf);
+            free(found_path);
+            free(foundfiles);
+            free(prefixed_path);
+            free(rom_filename);
+            printf("realloc failed in process_path.\n");
+            exit(1);
+        }
+        foundfiles = (char **)temp;
         foundfiles[numfiles - 1] = strdup(found_path);
     }
 
