@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* gdevdsp.c */
@@ -248,6 +248,9 @@ display_open(gx_device * dev)
         return ccode;
     dev = (gx_device *)ddev;
 
+    while(dev->parent)
+        dev = dev->parent;
+
     /* Make sure we have been passed a valid callback structure. */
     if ((ccode = display_check_structure(ddev)) < 0)
         return_error(ccode);
@@ -306,9 +309,12 @@ display_sync_output(gx_device * dev)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;	/* ignore the call */
     display_set_separations(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     (*(ddev->callback->display_sync))(ddev->pHandle, dev);
     return (0);
 }
@@ -321,9 +327,12 @@ display_output_page(gx_device * dev, int copies, int flush)
     gx_device_display *ddev = (gx_device_display *) dev;
     int code;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     display_set_separations(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     code = (*(ddev->callback->display_page))
                         (ddev->pHandle, dev, copies, flush);
 
@@ -338,7 +347,7 @@ display_close(gx_device * dev)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;	/* ignore the call since we were never properly opened */
 
     /* Tell caller that device is about to be closed. */
     (*(ddev->callback->display_preclose))(ddev->pHandle, dev);
@@ -346,6 +355,9 @@ display_close(gx_device * dev)
     /* Release memory. */
     display_free_bitmap(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     /* Tell caller that device is closed. */
     /* This is always the last callback */
     (*(ddev->callback->display_close))(ddev->pHandle, dev);
@@ -709,9 +721,13 @@ display_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return 0;		/* ignore -- needed for fillpage when device wasn't really opened */
     dev_proc(ddev->mdev, fill_rectangle)((gx_device *)ddev->mdev,
         x, y, w, h, color);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -726,9 +742,13 @@ display_copy_mono(gx_device * dev,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_mono)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h, zero, one);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -742,9 +762,13 @@ display_copy_color(gx_device * dev,
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_color)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -755,7 +779,7 @@ display_get_bits(gx_device * dev, int y, byte * str, byte ** actual_data)
 {
     gx_device_display *ddev = (gx_device_display *) dev;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
     return dev_proc(ddev->mdev, get_bits)((gx_device *)ddev->mdev,
         y, str, actual_data);
 }
@@ -1012,9 +1036,15 @@ display_put_params(gx_device * dev, gs_param_list * plist)
         /* We can resize this device while it is open, but we cannot
          * change the color format or handle.
          */
+
+        while(dev->parent) {
+            dev = dev->parent;
+            gx_update_from_subclass(dev);
+        }
+
         /* Tell caller we are about to change the device parameters */
         if ((*ddev->callback->display_presize)(ddev->pHandle, dev,
-            dev->width, dev->height, display_raster(ddev),
+            ddev->width, ddev->height, display_raster(ddev),
             ddev->nFormat) < 0) {
             /* caller won't let us change the size */
             /* restore parameters then return an error */
@@ -1023,11 +1053,17 @@ display_put_params(gx_device * dev, gs_param_list * plist)
             display_set_color_format(ddev, old_format);
             ddev->nFormat = old_format;
             ddev->pHandle = old_handle;
-            dev->width = old_width;
-            dev->height = old_height;
+            ddev->width = old_width;
+            ddev->height = old_height;
             return_error(gs_error_rangecheck);
         }
 
+        dev = (gx_device *)ddev;
+        while(dev->parent) {
+            dev = dev->parent;
+            gx_update_from_subclass(dev);
+        }
+
         display_free_bitmap(ddev);
 
         code = display_alloc_bitmap(ddev, dev);
@@ -1128,7 +1164,6 @@ static gx_color_index
 display_separation_encode_color(gx_device *dev, const gx_color_value colors[])
 {
     int bpc = ((gx_device_display *)dev)->devn_params.bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     gx_color_index color = 0;
     int i = 0;
     int ncomp = dev->color_info.num_components;
@@ -1152,7 +1187,6 @@ display_separation_decode_color(gx_device * dev, gx_color_index color,
     gx_color_value * out)
 {
     int bpc = ((gx_device_display *)dev)->devn_params.bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     int mask = (1 << bpc) - 1;
     int i = 0;
     int ncomp = dev->color_info.num_components;
@@ -1329,7 +1363,7 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
     int ccode;
     const gx_device_memory *mdproto;
     if (ddev->callback == NULL)
-        return 0;
+        return gs_error_Fatal;
 
     /* free old bitmap (if any) */
     display_free_bitmap(ddev);
@@ -1401,7 +1435,7 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
                 ? gx_max_color_value : 0;
         dev_proc(ddev, fill_rectangle)((gx_device *)ddev,
                  0, 0, ddev->width, ddev->height,
-                 ddev->procs.encode_color((gx_device *)ddev, cv));
+                 dev_proc(ddev, encode_color)((gx_device *)ddev, cv));
     }
 
     return ccode;
@@ -1471,6 +1505,9 @@ display_set_separations(gx_device_display *dev)
                            * 65535 / frac_1;
                 }
             }
+            while(dev->parent)
+                dev = (gx_device_display *)dev->parent;
+
             (*dev->callback->display_separation)(dev->pHandle, dev,
                 comp_num, name,
                 (unsigned short)c, (unsigned short)m,
