@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -57,6 +57,7 @@
 #include "ivmspace.h"
 #include "idisp.h"              /* for setting display device callback */
 #include "iplugin.h"
+#include "zfile.h"
 
 #ifdef PACIFY_VALGRIND
 #include "valgrind.h"
@@ -70,12 +71,7 @@
 gs_main_instance*
 get_minst_from_memory(const gs_memory_t *mem)
 {
-#ifdef PSI_INCLUDED
-    extern gs_main_instance *ps_impl_get_minst( const gs_memory_t *mem );
-    return ps_impl_get_minst(mem);
-#else
     return (gs_main_instance*)mem->gs_lib_ctx->top_of_system;
-#endif
 }
 
 /** construct main instance caller needs to retain */
@@ -93,12 +89,7 @@ gs_main_alloc_instance(gs_memory_t *mem)
         return NULL;
     memcpy(minst, &gs_main_instance_init_values, sizeof(gs_main_instance_init_values));
     minst->heap = mem;
-
-#   ifndef PSI_INCLUDED
     mem->gs_lib_ctx->top_of_system = minst;
-    /* else top of system is pl_universe */
-#   endif
-
     return minst;
 }
 
@@ -136,7 +127,6 @@ int
 gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
               int max_lib_paths)
 {
-    ref *paths;
     ref *array;
 
     /* Do platform-dependent initialization. */
@@ -146,25 +136,17 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
     gp_init();
 
     /* Initialize the imager. */
-#   ifndef PSI_INCLUDED
+
     /* Reset debugging flags */
 #ifdef PACIFY_VALGRIND
     VALGRIND_HG_DISABLE_CHECKING(gs_debug, 128);
 #endif
     memset(gs_debug, 0, 128);
     gs_log_errors = 0;  /* gs_debug['#'] = 0 */
-#   else
-    /* plmain settings remain in effect */
-#   endif
-    gp_get_usertime(minst->base_time);
+
+    gp_get_realtime(minst->base_time);
 
     /* Initialize the file search paths. */
-    paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
-                                        "lib_path array");
-    if (paths == 0) {
-        gs_lib_finit(1, gs_error_VMerror, minst->heap);
-        return_error(gs_error_VMerror);
-    }
     array = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
                                         "lib_path array");
     if (array == 0) {
@@ -187,44 +169,54 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 int
 gs_main_init1(gs_main_instance * minst)
 {
+    gs_dual_memory_t idmem;
+    name_table *nt = NULL;
+    int code = 0;
+
     if (minst->init_done < 1) {
-        gs_dual_memory_t idmem;
-        int code =
-            ialloc_init(&idmem, minst->heap,
-                        minst->memory_chunk_size, gs_have_level2());
+        code = ialloc_init(&idmem, minst->heap,
+                        minst->memory_clump_size, gs_have_level2());
 
         if (code < 0)
             return code;
         code = gs_lib_init1((gs_memory_t *)idmem.space_system);
         if (code < 0)
-            return code;
+            goto fail;
         alloc_save_init(&idmem);
         {
             gs_memory_t *mem = (gs_memory_t *)idmem.space_system;
-            name_table *nt = names_init(minst->name_table_size,
-                                        idmem.space_system);
+            nt = names_init(minst->name_table_size, idmem.space_system);
 
-            if (nt == 0)
-                return_error(gs_error_VMerror);
+            if (nt == 0) {
+                code = gs_note_error(gs_error_VMerror);
+                goto fail;
+            }
             mem->gs_lib_ctx->gs_name_table = nt;
-            code = gs_register_struct_root(mem, NULL,
+            code = gs_register_struct_root(mem, mem->gs_lib_ctx->name_table_root,
                                            (void **)&mem->gs_lib_ctx->gs_name_table,
                                            "the_gs_name_table");
             if (code < 0)
-                return code;
+                goto fail;
+            mem->gs_lib_ctx->client_check_file_permission = z_check_file_permissions;
         }
         code = obj_init(&minst->i_ctx_p, &idmem);  /* requires name_init */
         if (code < 0)
-            return code;
+            goto fail;
+        minst->init_done = 1;
         code = i_plugin_init(minst->i_ctx_p);
         if (code < 0)
-            return code;
-        code = i_iodev_init(minst->i_ctx_p);
+            goto fail;
+        code = i_iodev_init(&idmem);
         if (code < 0)
-            return code;
-        minst->init_done = 1;
+            goto fail;
     }
     return 0;
+
+fail:
+    names_free(nt);
+    if (minst->i_ctx_p == NULL)
+        ialloc_finit(&idmem);
+    return code;
 }
 
 /*
@@ -272,18 +264,18 @@ int gs_main_init2aux(gs_main_instance * minst) {
 
         /* Set up the array of additional initialization files. */
         make_const_string(&ifa, a_readonly | avm_foreign, gs_init_files_sizeof - 2, gs_init_files);
-        code = initial_enter_name("INITFILES", &ifa);
+        code = i_initial_enter_name(i_ctx_p, "INITFILES", &ifa);
         if (code < 0)
             return code;
 
         /* Set up the array of emulator names. */
         make_const_string(&ifa, a_readonly | avm_foreign, gs_emulators_sizeof - 2, gs_emulators);
-        code = initial_enter_name("EMULATORS", &ifa);
+        code = i_initial_enter_name(i_ctx_p, "EMULATORS", &ifa);
         if (code < 0)
             return code;
 
         /* Pass the search path. */
-        code = initial_enter_name("LIBPATH", &minst->lib_path.list);
+        code = i_initial_enter_name(i_ctx_p, "LIBPATH", &minst->lib_path.list);
         if (code < 0)
             return code;
 
@@ -299,7 +291,6 @@ int gs_main_init2aux(gs_main_instance * minst) {
         if ((code = display_set_callback(minst, minst->display)) < 0)
             return code;
 
-#ifndef PSI_INCLUDED
         if ((code = gs_main_run_string(minst,
                 "JOBSERVER "
                 " { false 0 .startnewjob } "
@@ -307,7 +298,6 @@ int gs_main_init2aux(gs_main_instance * minst) {
                 "ifelse", 0, &exit_code,
                 &error_object)) < 0)
            return code;
-#endif /* PSI_INCLUDED */
     }
     return 0;
 }
@@ -321,7 +311,7 @@ gs_main_init2(gs_main_instance * minst)
 
     if (code < 0)
         return code;
-    i_ctx_p = minst->i_ctx_p;
+
     code = gs_main_init2aux(minst);
     if (code < 0)
        return code;
@@ -333,7 +323,7 @@ gs_main_init2(gs_main_instance * minst)
        gx_device_printer *ppdev = (gx_device_printer *)pdev;
 
         if (minst->saved_pages_test_mode) {
-            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0)) {
+            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0)) {
                 /* no warning or error if saved-pages-test mode is used, just disable it */
                 minst->saved_pages_test_mode = false;  /* device doesn't support it */
             } else {
@@ -344,7 +334,7 @@ gs_main_init2(gs_main_instance * minst)
                         return code;
             }
         } else if (minst->saved_pages_initial_arg != NULL) {
-            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
                 outprintf(minst->heap,
                           "   --saved-pages not supported by the '%s' device.\n",
                           pdev->dname);
@@ -854,11 +844,27 @@ static char *gs_main_tempnames(gs_main_instance *minst)
     return tempnames;
 }
 
+static void
+gs_finit_push_systemdict(i_ctx_t *i_ctx_p)
+{
+    if (i_ctx_p == NULL)
+        return;
+    if (dsp == dstop ) {
+        if (ref_stack_extend(&d_stack, 1) < 0) {
+            /* zend() cannot fail */
+            (void)zend(i_ctx_p);
+        }
+    }
+    dsp++;
+    ref_assign(dsp, systemdict);
+}
+
 /* Free all resources and return. */
 int
 gs_main_finit(gs_main_instance * minst, int exit_status, int code)
 {
     i_ctx_t *i_ctx_p = minst->i_ctx_p;
+    gs_dual_memory_t dmem = {0};
     int exit_code;
     ref error_object;
     char *tempnames;
@@ -874,44 +880,72 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      */
     tempnames = gs_main_tempnames(minst);
 
-#ifndef PSI_INCLUDED
+    /* by the time we get here, we *must* avoid any random redefinitions of
+     * operators etc, so we push systemdict onto the top of the dict stack.
+     * We do this in C to avoid running into any other re-defininitions in the
+     * Postscript world.
+     */
+    gs_finit_push_systemdict(i_ctx_p);
+
     /* We have to disable BGPrint before we call interp_reclaim() to prevent the
      * parent rendering thread initialising for the next page, whilst we are
      * removing objects it may want to access - for example, the I/O device table.
      * We also have to mess with the BeginPage/EndPage procs so that we don't
      * trigger a spurious extra page to be emitted.
      */
-    if (minst->init_done >= 1) {
+    if (minst->init_done >= 2) {
         gs_main_run_string(minst,
-            "/systemdict .systemexec /begin .systemexec \
-            /BGPrint /GetDeviceParam .special_op \
+            "/BGPrint /GetDeviceParam .special_op \
             {{ <</BeginPage {pop} /EndPage {pop pop //false } \
               /BGPrint false /NumRenderingThreads 0>> setpagedevice} if} if \
-              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse end \
+              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse \
               .systemvar exec",
             0 , &exit_code, &error_object);
     }
-#endif
 
     /*
      * Close the "main" device, because it may need to write out
      * data before destruction. pdfwrite needs so.
      */
-    if (minst->init_done >= 1) {
+    if (minst->init_done >= 2) {
         int code = 0;
 
         if (idmemory->reclaim != 0) {
             code = interp_reclaim(&minst->i_ctx_p, avm_global);
 
             if (code < 0) {
-                emprintf1(minst->heap,
-                          "ERROR %d reclaiming the memory while the interpreter finalization.\n",
-                          code);
+                ref error_name;
+                if (tempnames)
+                    free(tempnames);
+
+                if (gs_errorname(i_ctx_p, code, &error_name) >= 0) {
+                    char err_str[32] = {0};
+                    name_string_ref(imemory, &error_name, &error_name);
+                    memcpy(err_str, error_name.value.const_bytes, r_size(&error_name));
+                    emprintf2(imemory, "ERROR: %s (%d) reclaiming the memory while the interpreter finalization.\n", err_str, code);
+                }
+                else {
+                    emprintf1(imemory, "UNKNOWN ERROR %d reclaiming the memory while the interpreter finalization.\n", code);
+                }
+#ifdef MEMENTO_SQUEEZE_BUILD
+                if (code != gs_error_VMerror ) return gs_error_Fatal;
+#else
                 return gs_error_Fatal;
+#endif
             }
             i_ctx_p = minst->i_ctx_p; /* interp_reclaim could change it. */
         }
-#ifndef PSI_INCLUDED
+
+        if (i_ctx_p->pgs != NULL && i_ctx_p->pgs->device != NULL &&
+            gx_device_is_null(i_ctx_p->pgs->device)) {
+            /* if the job replaced the device with the nulldevice, we we need to grestore
+               away that device, so the block below can properly dispense
+               with the default device.
+             */
+            int code = gs_grestoreall(i_ctx_p->pgs);
+            if (code < 0) return_error(gs_error_Fatal);
+        }
+
         if (i_ctx_p->pgs != NULL && i_ctx_p->pgs->device != NULL) {
             gx_device *pdev = i_ctx_p->pgs->device;
             const char * dname = pdev->dname;
@@ -925,25 +959,30 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                 /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
                 0 , &exit_code, &error_object);
             code = gs_closedevice(pdev);
-            if (code < 0)
-                emprintf2(pdev->memory,
-                          "ERROR %d closing %s device. See gs/psi/ierrors.h for code explanation.\n",
-                          code,
-                          dname);
+            if (code < 0) {
+                ref error_name;
+                if (gs_errorname(i_ctx_p, code, &error_name) >= 0) {
+                    char err_str[32] = {0};
+                    name_string_ref(imemory, &error_name, &error_name);
+                    memcpy(err_str, error_name.value.const_bytes, r_size(&error_name));
+                    emprintf3(imemory, "ERROR: %s (%d) on closing %s device.\n", err_str, code, dname);
+                }
+                else {
+                    emprintf2(imemory, "UNKNOWN ERROR %d closing %s device.\n", code, dname);
+               }
+            }
             rc_decrement(pdev, "gs_main_finit");                /* device might be freed */
             if (exit_status == 0 || exit_status == gs_error_Quit)
                 exit_status = code;
         }
-#endif
-    }
-    /* Flush stdout and stderr */
-    if (minst->init_done >= 2)
+
+      /* Flush stdout and stderr */
       gs_main_run_string(minst,
         "(%stdout) (w) file closefile (%stderr) (w) file closefile \
-        /systemdict .systemexec /begin .systemexec \
         serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemexec \
-        end",
+          systemdict /savedinitialgstate .forceundef",
         0 , &exit_code, &error_object);
+    }
     gp_readline_finit(minst->readline_data);
     i_ctx_p = minst->i_ctx_p;		/* get current interp context */
     if (gs_debug_c(':')) {
@@ -955,14 +994,17 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     if (minst->init_done >= 1) {
         gs_memory_t *mem_raw = i_ctx_p->memory.current->non_gc_memory;
         i_plugin_holder *h = i_ctx_p->plugin_list;
-        code = alloc_restore_all(idmemory);
+
+        dmem = *idmemory;
+        code = alloc_restore_all(i_ctx_p);
         if (code < 0)
             emprintf1(mem_raw,
                       "ERROR %d while the final restore. See gs/psi/ierrors.h for code explanation.\n",
                       code);
+        i_iodev_finit(&dmem);
         i_plugin_finit(mem_raw, h);
     }
-#ifndef PSI_INCLUDED
+
     /* clean up redirected stdout */
     if (minst->heap->gs_lib_ctx->fstdout2
         && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstdout)
@@ -970,7 +1012,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
         fclose(minst->heap->gs_lib_ctx->fstdout2);
         minst->heap->gs_lib_ctx->fstdout2 = (FILE *)NULL;
     }
-#endif
+
     minst->heap->gs_lib_ctx->stdout_is_redirected = 0;
     minst->heap->gs_lib_ctx->stdout_to_stderr = 0;
     /* remove any temporary files, after ghostscript has closed files */
@@ -982,9 +1024,10 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
         }
         free(tempnames);
     }
-#ifndef PSI_INCLUDED
     gs_lib_finit(exit_status, code, minst->heap);
-#endif
+
+    gs_free_object(minst->heap, minst->lib_path.container.value.refs, "lib_path array");
+    ialloc_finit(&dmem);
     return exit_status;
 }
 int
@@ -1014,36 +1057,34 @@ void
 print_resource_usage(const gs_main_instance * minst, gs_dual_memory_t * dmem,
                      const char *msg)
 {
-    ulong allocated = 0, used = 0;
+    ulong used = 0;		/* this we accumulate for the PS memories */
     long utime[2];
+    int i;
+    gs_memory_status_t status;
 
-    gp_get_usertime(utime);
-    {
-        int i;
+    gp_get_realtime(utime);
 
-        for (i = 0; i < countof(dmem->spaces_indexed); ++i) {
-            gs_ref_memory_t *mem = dmem->spaces_indexed[i];
+    for (i = 0; i < countof(dmem->spaces_indexed); ++i) {
+        gs_ref_memory_t *mem = dmem->spaces_indexed[i];
 
-            if (mem != 0 && (i == 0 || mem != dmem->spaces_indexed[i - 1])) {
-                gs_memory_status_t status;
-                gs_ref_memory_t *mem_stable =
-                    (gs_ref_memory_t *)gs_memory_stable((gs_memory_t *)mem);
+        if (mem != 0 && (i == 0 || mem != dmem->spaces_indexed[i - 1])) {
+            gs_ref_memory_t *mem_stable =
+                (gs_ref_memory_t *)gs_memory_stable((gs_memory_t *)mem);
 
-                gs_memory_status((gs_memory_t *)mem, &status);
-                allocated += status.allocated;
+            gs_memory_status((gs_memory_t *)mem, &status);
+            used += status.used;
+            if (mem_stable != mem) {
+                gs_memory_status((gs_memory_t *)mem_stable, &status);
                 used += status.used;
-                if (mem_stable != mem) {
-                    gs_memory_status((gs_memory_t *)mem_stable, &status);
-                    allocated += status.allocated;
-                    used += status.used;
-                }
             }
         }
     }
-    dmprintf4(minst->heap, "%% %s time = %g, memory allocated = %lu, used = %lu\n",
+    /* Now get the overall values from the heap memory */
+    gs_memory_status(minst->heap, &status);
+    dmprintf5(minst->heap, "%% %s time = %g, memory allocated = %lu, used = %lu, max_used = %lu\n",
               msg, utime[0] - minst->base_time[0] +
               (utime[1] - minst->base_time[1]) / 1000000000.0,
-              allocated, used);
+              status.allocated, used, status.max_used);
 }
 
 /* Dump the stacks after interpretation */
