@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -90,6 +90,7 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
                 break;
             }
         }
+        gs_free_object(gs_gstate_memory(igs), str, "seticc");
     } else {
         picc_profile = gsicc_profile_new(s, gs_gstate_memory(igs), NULL, 0);
         if (picc_profile == NULL)
@@ -116,8 +117,10 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     }
     picc_profile->num_comps = ncomps;
 
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
-    switch( picc_profile->data_cs ) {
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
+    switch (picc_profile->data_cs) {
         case gsCIEXYZ:
         case gsCIELAB:
         case gsRGB:
@@ -241,6 +244,9 @@ zset_outputintent(i_ctx_t * i_ctx_p)
 
     /* Get the device structure */
     code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+    if (code < 0)
+        return code;
+
     if (dev_profile == NULL) {
         code = gsicc_init_device_profile_struct(dev, NULL, 0);
         if (code < 0)
@@ -274,7 +280,9 @@ zset_outputintent(i_ctx_t * i_ctx_p)
         rc_decrement(picc_profile,"zset_outputintent");
         return -1;
     }
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
     switch (picc_profile->data_cs) {
         case gsCIEXYZ:
         case gsCIELAB:
@@ -503,7 +511,9 @@ znumicc_components(i_ctx_t * i_ctx_p)
         make_int(op, expected);
         return 0;
     }
-    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
+    picc_profile->data_cs =
+        gscms_get_profile_data_space(picc_profile->profile_handle,
+            picc_profile->memory);
 
     switch (picc_profile->data_cs) {
         case gsCIEXYZ:
